@@ -42,6 +42,7 @@
 #include <QtGui/QResizeEvent>
 #include <QtGui/QMouseEvent>
 #include <QtGui/QDesktopWidget>
+#include <QtGui/QCursor>
 #include <QtDBus/QDBusConnection>
 
 // include files for KDE
@@ -49,6 +50,9 @@
 #include <kcursor.h>
 #include <kdebug.h>
 #include <klocale.h>
+#ifdef QAccessibilityClient_FOUND
+#include <qaccessibilityclient/accessibleobject.h>
+#endif
 
 // include bitmaps for cursors
 static uchar left_ptr_bits[] = {
@@ -137,6 +141,11 @@ KMagZoomView::KMagZoomView(QWidget *parent, const char *name)
 
   if(m_fitToWindow)
     fitToWindow();
+
+#ifdef QAccessibilityClient_FOUND
+  //subscribe to focus events from registry
+  m_registry.subscribeEventListeners(QAccessibleClient::Registry::Focus | QAccessibleClient::Registry::TextCaretMoved);
+#endif
 }
 
 KMagZoomView::~KMagZoomView()
@@ -202,6 +211,29 @@ void KMagZoomView::followMouse(bool follow)
   }
 }
 
+#ifdef QAccessibilityClient_FOUND
+
+void KMagZoomView::followBoth(bool follow)
+{
+    m_followBoth = follow;
+    if(follow){
+        m_followMouse = true;
+        m_followFocus = false;
+        setVerticalScrollBarPolicy (Qt::ScrollBarAlwaysOff);
+        setHorizontalScrollBarPolicy (Qt::ScrollBarAlwaysOff);
+
+        connect(&m_registry, SIGNAL(focusChanged(QAccessibleClient::AccessibleObject)),
+                this, SLOT(focusChanged(QAccessibleClient::AccessibleObject)));
+        connect(&m_registry, SIGNAL(textCaretMoved(QAccessibleClient::AccessibleObject,int)),
+                this, SLOT(focusChanged(QAccessibleClient::AccessibleObject)));
+    } else {
+        setVerticalScrollBarPolicy (Qt::ScrollBarAlwaysOn);
+        setHorizontalScrollBarPolicy (Qt::ScrollBarAlwaysOn);
+
+        disconnect(this, SLOT(focusChanged(QAccessibleClient::AccessibleObject)));
+    }
+}
+
 /**
  * This function will set/reset keyboard focus following of grab window.
  */
@@ -214,22 +246,31 @@ void KMagZoomView::followFocus(bool follow)
   if(follow) {
     setVerticalScrollBarPolicy (Qt::ScrollBarAlwaysOff);
     setHorizontalScrollBarPolicy (Qt::ScrollBarAlwaysOff);
-    if(QDBusConnection::sessionBus().isConnected())
-        QDBusConnection::sessionBus().connect(QLatin1String( "org.kde.kaccessibleapp" ), QLatin1String( "/Adaptor" ), QLatin1String( "org.kde.kaccessibleapp.Adaptor" ), QLatin1String( "focusChanged" ), this, SLOT(focusChanged(int,int,int,int,int,int)));
+
+    connect(&m_registry,SIGNAL(focusChanged(QAccessibleClient::AccessibleObject)),
+            this, SLOT(focusChanged(QAccessibleClient::AccessibleObject)));
+    connect(&m_registry, SIGNAL(textCaretMoved(QAccessibleClient::AccessibleObject,int)),
+            this, SLOT(focusChanged(QAccessibleClient::AccessibleObject)));
   } else {
     setVerticalScrollBarPolicy (Qt::ScrollBarAlwaysOn);
     setHorizontalScrollBarPolicy (Qt::ScrollBarAlwaysOn);
-    if(QDBusConnection::sessionBus().isConnected())
-        QDBusConnection::sessionBus().disconnect(QLatin1String( "org.kde.kaccessibleapp" ), QLatin1String( "/Adaptor" ), QLatin1String( "org.kde.kaccessibleapp.Adaptor" ), QLatin1String( "focusChanged" ), this, SLOT(focusChanged(int,int,int,int,int,int)));
+
+    disconnect(this, SLOT(focusChanged(QAccessibleClient::AccessibleObject)));
   }
 }
 
-void KMagZoomView::focusChanged(int px, int py, int rx, int ry, int rwidth, int rheight)
+void KMagZoomView::focusChanged(const QAccessibleClient::AccessibleObject &object)
 {
-    //kDebug() << "px=" << px << "py=" << py << "rx=" << rx << "ry=" << ry << "rwidth=" << rwidth << "rheight=" << rheight;
-    m_oldFocus = (px >= 0 && py >= 0) ? QPoint(px, py) : QPoint( rx + qMax(0, (qMin(width(), rwidth)/2)-60), ry + qMax(0, (qMin(height(), rheight)/2)-60) );
+    m_oldFocus = object.focusPoint();
+    if(m_followBoth && !m_selRect.contains(m_oldFocus)) {
+        QCursor::setPos(m_oldFocus);
+        m_followFocus = true;
+        m_followMouse = false;
+    }
 }
 
+#endif
+
 /**
  * Called when the widget is hidden. Stop refresh when this happens.
  */
@@ -884,9 +925,15 @@ void KMagZoomView::grabFrame()
     if(m_followMouse) {
         // set new center to be the current mouse position
         newCenter = QCursor::pos();
+#ifdef QAccessibilityClient_FOUND
     } else if(m_followFocus) {
         // set the new center to the current keyboard cursor position
         newCenter = m_oldFocus;
+        if(m_followBoth) {
+            m_followFocus=false;
+            m_followMouse=true;
+        }
+#endif
     }
 
     // make sure the mouse position is not taking the grab window outside
